#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Aug 30 10:58:54 2019

@author: poessel
Markus Poessel, December 2019
poessel@hda-hd.de
as supplementary material for arXiv:TBDTBDTBD
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
import ephem
from astropy import constants as const
from astropy import units as u

#%% 

# Preparations: Time delay factor
# prefactor in Microseconds
timeUnit = float(const.GM_sun/np.power(const.c,3)/(1e-6*u.s))

conjTime = 25591.35215


#%%

# Here, the ephemeris data necessary for the Shapiro plot is collected:
dateRange = np.linspace(conjTime-300,conjTime+300,10000)

elongCollector=[]
dSECollector =[]
dEVCollector=[]
dSVCollector=[]

for thisDate in dateRange:
    venus=ephem.Venus(thisDate)
    elong=venus.elong
    elongCollector.append(elong)
    dSV=venus.sun_distance
    dSVCollector.append(dSV)
    dEV=venus.earth_distance
    dEVCollector.append(dEV)
    sun=ephem.Sun(thisDate)
    dSE = sun.earth_distance
    dSECollector.append(dSE)

# I use this data to generate the coordinate values I need for my time delay prediction:   
eta = [ np.abs(thisdSE*np.sin(thisElong)) for thisdSE, thisElong in zip(dSECollector,elongCollector)]
xVPos = np.array([ np.sqrt(thisDSV**2-thisEta**2) for  thisDSV,thisEta in zip(dSVCollector,eta)  ])
xVNeg = np.array([ -np.sqrt(thisDSV**2-thisEta**2) for  thisDSV,thisEta in zip(dSVCollector,eta)  ])

halfWay = int(0.5*len(xVPos))
firstMin = np.argmin(xVPos[:halfWay])
secondMin= np.argmin(xVPos[halfWay:])+halfWay

# Use this to calculate the Venus values in the chosen coordinate system:
xV=np.concatenate((xVPos[0:firstMin],xVNeg[firstMin:secondMin],xVPos[secondMin:]))
# Next, calculate the values xE
xE = [ np.sqrt(thisDSE**2-thisEta**2) for  thisDSE,thisEta in zip(dSECollector,eta)  ]
#... and calculate the argument of the logarithm in the Shapiro expression
shapArg = np.array([ (thisxE+thisDSE)/(thisxV + thisDSV) for thisxE, thisDSE, thisxV, thisDSV in zip(xE, dSECollector, xV, dSVCollector) ])

def delayDoubleNewton(xV,xE,dSE,dSV):
    return 4*timeUnit*np.log((xE+dSE)/(xV+dSV))

# This here defines the correct relativistic Shapiro time delay:
shapiroDelayDoubleN = [ delayDoubleNewton(thisxV,thisxE,thisDSE,thisDSV) for thisxV,thisxE,thisDSE,thisDSV in zip(xV, xE, dSECollector,dSVCollector)]

#%%

# For comparison purposes, this calculates the delay for the pseudo-postNewtonian
# calculation in Schwarzschild coordinates:
def delaySchwarzSchildCoord(xV,xE,dSE,dSV):
    return delayDoubleNewton(xV,xE,dSE,dSV)+2*timeUnit*(xV/dSV-xE/dSE)

shapiroDelaySSC = [ delaySchwarzSchildCoord(thisxV,thisxE,thisDSE,thisDSV) for thisxV,thisxE,thisDSE,thisDSV in zip(xV, xE, dSECollector,dSVCollector)]


#%%
    
def delayFormula(dSE,dSV,dEV,eta):
    if dEV > np.sqrt(dSE**2 -eta**2):
        return (4*timeUnit*np.log((np.sqrt(dSE**2-eta**2)+dSE)*(np.sqrt(dSV**2-eta**2)+dSV)/(eta**2))-2*timeUnit*(np.sqrt(1-eta**2/dSE**2)+np.sqrt(1-eta**2/dSV**2)))
    else:
        return (4*timeUnit*np.log((np.sqrt(dSE**2-eta**2)+dSE)/(np.sqrt(dSV**2-eta**2)+dSV))-2*timeUnit*(np.sqrt(1-eta**2/dSE**2)-np.sqrt(1-eta**2/dSV**2)))

shapiroDelay = [ delayFormula(thisDSE,thisDSV,thisDEV,thisEta) for thisDSE, thisDSV, thisDEV, thisEta in zip(dSECollector,dSVCollector,dEVCollector,eta)]

def delayFormula2(dSE,dSV,dEV,eta):
    return 4*timeUnit*np.log((dSE+dSV+dEV)/(dSE+dSV-dEV))

shapiroDelay2 = [ delayFormula2(thisDSE,thisDSV,thisDEV,thisEta) for thisDSE, thisDSV, thisDEV, thisEta in zip(dSECollector,dSVCollector,dEVCollector,eta)]

def delayFormulaN(dSE,dSV,dEV,eta):
      if dEV > np.sqrt(dSE**2 -eta**2):
        return (2*timeUnit*np.log((np.sqrt(dSE**2-eta**2)+dSE)*(np.sqrt(dSV**2-eta**2)+dSV)/(eta**2)))
      else:
        return (2*timeUnit*np.log((np.sqrt(dSE**2-eta**2)+dSE)/(np.sqrt(dSV**2-eta**2)+dSV)))
  

shapiroDelayN = [ delayFormulaN(thisDSE,thisDSV,thisDEV,thisEta) for thisDSE, thisDSV, thisDEV, thisEta in zip(dSECollector,dSVCollector,dEVCollector,eta)]

def delayFormula64(dSE,dSV,dEV,eta):
    if dEV > np.sqrt(dSE**2 -eta**2):
        return 4*timeUnit*np.log((np.sqrt(dSV**2-eta**2) +dSV)/(dSE-np.sqrt(dSE**2-eta**2)))-2*timeUnit*(np.sqrt(1-(eta/dSV)**2)+2*np.sqrt(1-(eta/dSE)**2)+np.sqrt(dSV**2-eta**2)/dSE**2)
    else:
        return 4*timeUnit*np.log((-np.sqrt(dSV**2-eta**2) +dSV)/(dSE-np.sqrt(dSE**2-eta**2)))-2*timeUnit*(-np.sqrt(1-(eta/dSV)**2)+2*np.sqrt(1-(eta/dSE)**2)-np.sqrt(dSV**2-eta**2)/dSE**2)
    
        
shapiroDelay64 = [ delayFormula64(thisDSE,thisDSV,thisDEV,thisEta) for thisDSE, thisDSV, thisDEV, thisEta in zip(dSECollector,dSVCollector,dEVCollector,eta)]


shapiroAux = [4*thisDSE*thisDSV/thisEta for thisDSE, thisDSV, thisEta in zip(dSECollector,dSVCollector,eta)]
plt.clf()
plt.plot(dateRange-conjTime,shapiroDelay)
plt.plot(dateRange-conjTime,shapiroDelay2)
#plt.plot(dateRange-conjTime,shapiroAux)
#plt.plot(dateRange-conjTime,eta)


#%%

# Abgelesen aus Shapiro1971, Fig. 1

################ y coordinates ##########################
# Reconstructing y coordinates
yMarkings = np.array([200,180,160,140,120,100,80,60,40,20,0,0,200])
yPixel    = [173.0,537.0,892.0,1237.0,1593.7,1944.3,2294.0,2652.0,3012.0,3364.3,3722.3,3708.3,151.3]
xPixel    = [601.3,603.0,605.7,608,610.3,610.7,610.7,612.7,615,615.3,616.7,5024.3,4998.7]

by,ay = np.polyfit(yPixel,yMarkings,1)

# Check for y axis
plt.clf()
plt.scatter(yPixel,yMarkings)
x=np.linspace(100,3800,370)
y= by*x+ay
plt.plot(x,y)

# Reconstructing y coordinates, now in 2d:

def yFunc(X,a,b,c):
    return X[1]*b +X[0]*c +a

XX=np.array(list(zip(xPixel,yPixel))).T

popt, pcov = curve_fit(yFunc, XX, yMarkings)

aay,bby,ccy=popt

def yMap(x,y):
    return aay+bby*y+ccy*x

################# x coordinates ##########################
    
xMarkings = np.array([-300,-250,-200,-150,-100,-50,0,50,100,150,200,250,300,324.4,-300])
xPixel = [617,969,1322.7,1673.7,2027.0,2378.3,2731.3,3087.7,3438.3,3792.0,4145.3,4499.0,4852.3,4999,602]
yPixel = [3722.7,3721.7,3720,3719.0,3717.3,3717.0,3716.3,3716.0,3714.7,3712.3,3712.0,3709.7,3709.3,151,173]

XX = np.array(list(zip(xPixel,yPixel))).T

def xFunc(X,a,b,c):
    return X[0]*b +X[1]*c +a


popt, pcov = curve_fit(xFunc, XX, xMarkings)
aax,bbx,ccx= popt

x = np.linspace(600,5000,100)
y = bbx*x + aax + ccx*3722

plt.clf()
plt.scatter(xPixel,xMarkings)
plt.plot(x,y)

def xMap(x,y):
    return bbx*x + aax + ccx*y

#%%
    
##### Arecibo data in px from Shapiro1971-figure.png
# This is the same as in the data files, but written as Python arrays. 
# (in fact, an earlier version of this script was used to generate those data files)
# PN7 38ff
    
areciboPX = [700.5,756,793.5,810,825,903,940.5,960,1047,1072.5,1108.5,1318.5,
             1341,1372.5,1389,1435.5,1488,1513.5,1590.0,1639.5,1686,1755,1777.5,
             1807.5,1827,1878,1933.5,1959,1986,3186,3252,3294,3307.5,3361.5,3384,3412.5,
             3483,3538.5,3555,3580.5,3660,3751.5,3769.5,3784.5,3816,3838.5,3858,
             3876,3910.5,3957,3996,4035,4051.5,4078.5,4102.5,4123.5]

areciboPY = [3619.5,3529.5,3625.5,3555,3600,3585,3489,3510,3445.5,3381,3420,3246,
             3342,3337.5,3379.5,3235.5,3300,3237,3172.5,3061.5,2989.5,2989.5,3022.5,2967,
             2935.5,3021,2920.5,2955,2976,2431.5,2553,2554.5,2688,2817,2797.5,2841,2746.5,
             2946,3006,2985,2914.5,3051,3019.5,3063,3075,3181.5,3129,3078,3198,3171,
             3243,3199.5,3253.5,3294,3319.5,3351]

areciboPEmax=[3537.5,3402,3537,3467,3512,3491,3386,3420,3359,3281,3300,3120,3266,3248,
              3284,3140,3213,3153,3083,2973,2898,2901,2945,2841,2808,2906,2798,2846,
              2855,2163,2285,2463,2591,2645,2709,2755,2667,2855,2874,2894,2819,2967,
              2936,2973,2985,3084,3042,2982,3108,3081,3164,3110,3200,3239,3263,3293]

areciboPEmin=[3717,3651,3711,3642,3683,3669,3566,3600,3534,3470,3542,3375,3437,3423,
              3461,3320,3387,3327,3252,3131,3077,3077,3093,3096,3054,3149,3047,3078,
              3102,2691,2817,2637,2774,2995,2885,2936,2838,3032,3125,3069,3023,3150,
              3115,3150,3161,3264,3215,3161,3288,3258,3324,3287,3309,3345,3374,3404]

##### Haystack data in px from Shapiro1971-figure.png
# PN7 38ff

haystackPX =[646.5,675,880.5,1002,1015.5,1066.5,1089,1141.5,1182,1216.5,1243.5,1261.5,1282.5,1311,
             1420.5,1477.5,1542,1572,1627.5,1666.5,1717.5,1779,2308.5,2593.5,2616,2637,
             2661,2721,2742,2775,2788.5,3057,3094.5,3147,3648,3714,3751,3786,3843,
             3859.5,3910.5,3949.5,4192.5,4230,4257,4285.5,4479,4549.5,4602,4644]

haystackPY = [3604.5,3612,3510,3558.0,3457.5,3301.5,3402,3385.5,3432,3363,3333,3394.5,
              3429,3310.5,3154.5,3348,3139.5,3127.5,3096,3129,3097.5,3100.5,2275.5,
              1479,1507.5,1021.5,876,471,652.5,784.5,889.5,2218.5,2269.5,2370,
              2845.5,3195,3091.5,3105,3088.5,3265.5,3235.5,3295.5,3300,3357,
              3355.5,3388.5,3486,3516,3555,3592.5]

haystackPEmax = [3572,3512,3473,3525,3419,3263,3368,3348,3405,3281,3281,3281,3311,3227,
                 3069,3231,2952,3013.5,3012,2936,2969,2942,2004,1211,1245,765,618,
                 285,380,617,633,2094,2000,2201.5,2667,3069,3003,3018,2970,3101,
                 3062,3218,3263,3315,3315,3356,3447,3479,3512,3557]

haystackPEmin = [3647,3686,3546,3599,3494,3339,3443,3435,3467,3446,3375,3521,3557,3399,
                 3240,3465,3309,3264,3183,3287,3222,3270,2526,1734,1821,1286,1134,
                 653,908,998,1155,2334,2531,2549,3024,3321,3212,3191,3213,3438,3408,
                 3378,3338,3392,3395,3425,3527,3554,3593,3633]



areciboX = np.array([xMap(xx,yy) for xx,yy in zip(areciboPX,areciboPY)])
areciboY = np.array([yMap(xx,yy) for xx,yy in zip(areciboPX,areciboPY)])
areciboErrMin = [yMap(xx,yy)-yMap(xx,ymin) for xx,yy,ymin in zip(areciboPX,areciboPY,areciboPEmin)]
areciboErrMax = [yMap(xx,ymax)-yMap(xx,yy) for xx,yy,ymax in zip(areciboPX,areciboPY,areciboPEmax)]
areciboErr=[areciboErrMin,areciboErrMax]

haystackX = np.array([xMap(xx,yy) for xx,yy in zip(haystackPX,haystackPY)])
haystackY = np.array([yMap(xx,yy) for xx,yy in zip(haystackPX,haystackPY)])
haystackErrMin = [yMap(xx,yy)-yMap(xx,ymin) for xx,yy,ymin in zip(haystackPX,haystackPY,haystackPEmin)]
haystackErrMax = [yMap(xx,ymax)-yMap(xx,yy) for xx,yy,ymax in zip(haystackPX,haystackPY,haystackPEmax)]
haystackErr=[haystackErrMin,haystackErrMax]
#%%

plt.clf()
plt.figure(figsize=(12,8))
plt.errorbar(areciboX,areciboY,yerr=areciboErr,marker='o',ms=4,capsize=2.5,linestyle='None',label='Arecibo')
plt.errorbar(haystackX,haystackY,yerr=haystackErr,marker='o',ms=4,capsize=2.5,linestyle='None',label='Haystack')
#plt.plot(dateRange-conjTime,shapiroDelay2,label='Shapiro time delay')
#plt.plot(dateRange-conjTime,shapiroDelay64,label='Shapiro 1964')
#plt.plot(dateRange-conjTime,shapiroDelay,label='Approximated delay')
#plt.plot(dateRange-conjTime,shapiroDelayN,label='Newtonian time delay')
plt.plot(dateRange-conjTime,shapiroDelayDoubleN,label='ART prediction from ephemeris')
plt.legend(fontsize=14)
plt.xlim(-300,320)
plt.ylim(0,210)
plt.gca().tick_params(axis='both', which='major', labelsize=14)
plt.gca().tick_params(axis='both', which='minor', labelsize=14)
plt.xlabel('Days from superior conjunction',fontsize=14)
plt.ylabel(u'Time delay in $\mu$s',fontsize=14)
plt.text(-250,175,'Superior conjunction of Venus',fontsize=14)
plt.text(-250,165,'1970/1/24 20:27:06',fontsize=14)
plt.text(-250,100,u'$\gamma=1.03\pm 0.04$',fontsize=16)

plt.savefig('shapiro-data.pdf',bbox_inches='tight')

#%%

plt.clf()
plt.figure(figsize=(12,8))
plt.errorbar(areciboX,areciboY,yerr=areciboErr,marker='o',ms=4,capsize=2.5,linestyle='None',label='Arecibo')
plt.errorbar(haystackX,haystackY,yerr=haystackErr,marker='o',ms=4,capsize=2.5,linestyle='None',label='Haystack')
#plt.plot(dateRange-conjTime,shapiroDelay2,label='Shapiro time delay')
#plt.plot(dateRange-conjTime,shapiroDelay,label='Approximated delay')
plt.plot(dateRange-conjTime,shapiroDelayDoubleN,label='ART')
#plt.plot(dateRange-conjTime,shapiroDelay64,label='Schwarzschild coords')
plt.plot(dateRange-conjTime,shapiroDelaySSC,label='Schwarzschild coords')
plt.plot(dateRange-conjTime,shapiroDelayN,label='Newtonian time delay')
plt.legend(fontsize=14)
plt.xlim(-300,320)
plt.ylim(0,210)
plt.gca().tick_params(axis='both', which='major', labelsize=14)
plt.gca().tick_params(axis='both', which='minor', labelsize=14)
plt.xlabel('Days from superior conjunction',fontsize=14)
plt.ylabel(u'Time delay in $\mu$s',fontsize=14)
plt.text(-250,175,'Superior conjunction of Venus',fontsize=14)
plt.text(-250,165,'1970/1/24 20:27:06',fontsize=14)
#plt.axhline(178)
#plt.axhline(158)

plt.savefig('shapiro-data-more.pdf',bbox_inches='tight')

#%%

plt.clf()
plt.figure(figsize=(12,8))
plt.errorbar(areciboX,areciboY,yerr=areciboErr,color='k',marker='o',ms=4,capsize=2.5,linestyle='None',label='Arecibo')
plt.errorbar(haystackX,haystackY,yerr=haystackErr,color='k',fmt='o',mfc='w',ms=4,capsize=2.5,linestyle='None',label='Haystack')
#plt.plot(dateRange-conjTime,shapiroDelay2,label='Shapiro time delay')
#plt.plot(dateRange-conjTime,shapiroDelay,label='Approximated delay')
plt.plot(dateRange-conjTime,shapiroDelayDoubleN,'k',linestyle='dashed',label='ART prediction')
#plt.plot(dateRange-conjTime,shapiroDelay64,label='Schwarzschild coords')
#plt.plot(dateRange-conjTime,shapiroDelaySSC,label='Schwarzschild coords')
#plt.plot(dateRange-conjTime,shapiroDelayN,label='Newtonian time delay')
plt.legend(fontsize=14)
plt.xlim(-300,320)
plt.ylim(0,210)
plt.gca().tick_params(axis='both', which='major', labelsize=14)
plt.gca().tick_params(axis='both', which='minor', labelsize=14)
plt.xlabel('Days from superior conjunction',fontsize=14)
plt.ylabel(u'Time delay in $\mu$s',fontsize=14)
plt.text(-250,175,'Superior conjunction of Venus',fontsize=14)
plt.text(-250,165,'1970/1/24 20:27:06',fontsize=14)
#plt.axhline(178)
#plt.axhline(158)

plt.savefig('shapiro-data-bw.pdf',bbox_inches='tight')
